var Font = Quill.import('formats/font');

var quill = new Quill('#editor', {
    modules: {}
});

// Creates and inserts a css class. Will be needed to support fonts
var createClass = function (name, rules) {
    var style = document.createElement('style');
    style.type = 'text/css';
    document.getElementsByTagName('head')[0].appendChild(style);

    if (!(style.sheet || {}).insertRule) {
        (style.styleSheet || style.sheet).addRule(name, rules);
    }
    else {
        style.sheet.insertRule(name + "{" + rules + "}", 0);
    }
};

var registerFonts = function(fonts) {
    for(var i = 0; i < fonts.length; i++) {
        var fixedFontName = fonts[i].replace(/_/g, " ");

        // first font will be the default
        if(i == 0) {
            createClass("#editor", "font-family: '" + fixedFontName + "';");
        }
        else {
            createClass(".ql-font-" + fonts[i], "font-family: '" + fixedFontName + "';");
        }
    }

    Font.whitelist = fonts;
    Quill.register(Font, true);
};

var getSelectionBounds = function() {
    var range = quill.getSelection();
    return quill.getBounds(range.index, range.height)
}

var getScrollTop = function() {
    return document.documentElement.scrollTop || document.body.scrollTop;
}

var getClientHeight = function() {
    return document.documentElement.clientHeight || document.body.clientHeight;
}

var restoreScroll = function() {
    var range = quill.getSelection();
    var bounds = quill.getBounds(range.index, range.height);
    var clientHeight = getClientHeight();
    var scrollTop = getScrollTop();
    if (bounds.top < scrollTop) {
        window.scrollTo(0, bounds.top)
    } else if (bounds.top + bounds.height + 8 > scrollTop + clientHeight) {
        window.scrollTo(0, bounds.top + bounds.height - clientHeight + 8)
    }
}

var insertEmbed = function(type, value, source) {
    var range = quill.getSelection();
    var length = quill.getLength();
    var result = quill.insertEmbed(range.index, type, value, source);
    var newLength = quill.getLength();
    if (newLength > length) {
        var newIndex = range.index + newLength - length;
        quill.insertText(newIndex, "\n");
        quill.setSelection(newIndex + 1, 0);
    }
    return result
}

var setContents = function(delta, source) {
    var range = quill.getSelection();
    var result = quill.setContents(delta, source)
    quill.setSelection(range.index, range.length);
    return result
}

var getHtml = function(delta) {
    var temp =new Quill(document.createElement("div"));
    temp.setContents(delta);
    return temp.root.innerHTML;
}

quill.on('selection-change', function(range, oldRange, source) {
    if(range) {
        Android.onSelectionChange(range.index, range.length, oldRange == null ? -1 : oldRange.index,
            oldRange == null ? -1 : oldRange.length, source);
    }
});

quill.on('text-change', function(delta, oldDelta, source) {
    Android.onTextChange(delta, oldDelta, source);
    var length = quill.getLength()
    var maxLength = quill.root.getAttribute("data-max-length")
    if (length > maxLength) {
        quill.deleteText(maxLength, length-maxLength)
    }
});